//
// MyView.m
// Core Animation Screen Cast
//
// Created by Bill Dudney on 05/01/08.
//
// Copyright 2008 Gala Factory
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

#import "MyView.h"
#import <QuartzCore/QuartzCore.h>
#import "CGColorHolder.h"
#import "MyDelegate.h"
#import "MyLayoutManager.h"

#define SCALE 0.45f
#define PIC_PER_ROW 2
#define Y_OFFSET -15.0f
#define X_OFFSET 25.0f

@interface MyView (Private)

- (void)addImageSublayers;

- (CGImageRef)loadImage:(NSString*)name;
- (CGRect)bestFitForImage:(CGImageRef) image;

@end

@implementation MyView

- (void)awakeFromNib {
	self.layer = [CALayer layer];
	self.wantsLayer = YES;
	self.layer.backgroundColor = [CGColorHolder black];
	self.layer.layoutManager = [MyLayoutManager layoutManager];
	[self addImageSublayers];
}

- (BOOL)acceptsFirstResponder {
	return YES;
}

- (void)moveUp:(id)sender {
	NSArray *sublayers = [self.layer sublayers];
	NSUInteger sublayerCount = [sublayers count];
	CALayer *sublayer = [sublayers objectAtIndex:sublayerCount - 1];
	[sublayer removeFromSuperlayer];
	[self.layer insertSublayer:sublayer atIndex:0];
}

- (void)moveDown:(id)sender {
	NSArray *sublayers = [self.layer sublayers];
	NSUInteger sublayerCount = [sublayers count];
	CALayer *sublayer = [sublayers objectAtIndex:0];
	[sublayer removeFromSuperlayer];
	[self.layer insertSublayer:sublayer atIndex:sublayerCount - 1];
}

@end

@implementation MyView (Private)

- (void)addImageSublayers {
	NSArray *imageNames = [NSArray arrayWithObjects:@"GettyMuseum", 
						   @"CreepyHollywoodHotel", @"SunThroughSnowClouds", 
						   @"TenMileRange", nil];
	for(int i = 0;i < [imageNames count]; i++) {
		NSString *imageName = [imageNames objectAtIndex:i];
		CALayer *sublayer = [CALayer layer];
		sublayer.contents = (id)[self loadImage:imageName];
		sublayer.name = imageName;
		sublayer.bounds = CGRectMake(0.0f, 0.0f, NSWidth(self.bounds) * SCALE, 
									 NSHeight(self.bounds) * SCALE);
		sublayer.position = CGPointMake(NSMidX(self.bounds), NSMidY(self.bounds));
		sublayer.contentsGravity = kCAGravityResizeAspectFill;
		sublayer.masksToBounds = YES;
		[self.layer addSublayer:sublayer];
	}
}	

- (CGRect) bestFitForImage:(CGImageRef)image {
	CGSize size = CGSizeMake(NSWidth([self bounds]) * SCALE, 
							NSHeight([self bounds]) * SCALE);
	CGFloat imageWidth = CGImageGetWidth(image);
	CGFloat imageHeight = CGImageGetHeight(image);
	CGFloat scale = 0.0f;
	if(size.width / imageWidth < size.height / imageHeight) {
		scale = size.width / imageWidth;
	} else {
		scale = size.height / imageHeight;
	}
	CGRect rect = {CGPointZero, {scale * imageWidth, scale * imageHeight}};
	// if we have a portrait image we want to center it on the width
	if(rect.size.width < size.width) {
		rect.origin.x = (size.width - rect.size.width)/2.0f;
	}
	return rect;
}

- (CGImageRef)loadImage:(NSString*)name {
	CGImageRef image = NULL;
	NSBundle *bun = [NSBundle mainBundle];
	NSString *path = [bun pathForResource:name ofType:@"jpg"];
	NSURL * url = [NSURL fileURLWithPath:path];
	CGImageSourceRef imageSource = CGImageSourceCreateWithURL((CFURLRef)url, NULL);
	if(imageSource != NULL) {
		image = CGImageSourceCreateImageAtIndex(imageSource, 0, NULL);
		CFRelease(imageSource);
	} else {
		NSLog(@"Could not load image");
		return NULL;
	}
	
	if(NULL != image) {
		CGRect rect = [self bestFitForImage:image];
		CGColorSpaceRef colorSpace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
		CGContextRef context = CGBitmapContextCreate(NULL, rect.size.width, 
													 rect.size.height, 8, 0, 
													 colorSpace, 
													 kCGImageAlphaPremultipliedFirst); 
		CGContextSetInterpolationQuality(context, kCGInterpolationHigh);
		rect.origin = CGPointZero;
		CGContextDrawImage(context, rect, image);
		CGContextFlush(context);
		CGImageRef scaledImage = CGBitmapContextCreateImage(context);
		CGContextRelease (context);
		CGImageRelease(image);
		image = NULL;
		image = scaledImage;
	}
	
	return image;
}

@end
